package MT::Plugin::Quickedit;
use strict;

use MT;
use MT::Entry;
use MT::Page;
use MT::FileInfo;
use MT::Util qw( encode_html );
our $VERSION = "0.1";

@MT::Plugin::Quickedit::ISA = qw( MT::Plugin );

my $plugin = new MT::Plugin::Quickedit( {
    name => 'Quickedit',
    version => $VERSION,
    description => 'Easy edit entry.',
    author_name => 'Alfasado,Inc',
    author_link => 'http://alfasado.net/',
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        applications => {
            cms => {
                methods => {
                    quickedit =>
                        'MT::Plugin::Quickedit::_quickedit',
                    quicksave =>
                        'MT::Plugin::Quickedit::_quicksave',
                },
            },
        },
   } );
}

sub _quicksave {
    my $app = shift;
    my $permalink = $app->param( 'permalink' );
    my $entry_id = $app->param( 'id' );
    my $title = $app->param( 'title' );
    my $text = $app->param( 'text' );
    my $class = $app->param( 'class' );
    my $e = MT->model( $class )->load( $entry_id );
    unless ( defined $e ) {
        return $app->trans_error( 'Invalid request.' );
    }
    unless ( _can_post( $app->user, $e->blog_id, $e ) ) {
        return $app->trans_error( 'Permission denied.' );
    }
    $e->title( $title );
    $e->text( $text );
    $e->save or die $e->errstr;
    MT::Util::start_background_task(
        sub { $app->rebuild_entry( Entry => $e,
                                   BuildDependencies => 1 );
            }
    );
    $app->redirect( $permalink );
}

sub _quickedit {
    my $app = shift;
    my $permalink = $app->param( 'permalink' );
    my $url = $permalink;
    $url =~ s!^https*://.*?(/.*$)!$1!;
    my @classes = ['Individual', 'Page']; 
    my $finfos = MT::FileInfo->load_iter( { url => $url,
                                            archive_type => \@classes, });
    my $e;
    while ( my $fi = $finfos->() ) {
        if ( $fi->entry_id ) {
            $e = MT::Entry->load( $fi->entry_id );
            if ( defined $e ) {
                if ( $permalink eq $e->permalink ) {
                    last;
                }
            }
        }
    }
    if ( defined $e ) {
        unless ( _can_post( $app->user, $e->blog_id, $e ) ) {
            return $app->trans_error( 'Permission denied.' );
        }
        my $class = $e->class;
        my $at;
        if ( $class eq 'entry' ) {
             $at = 'Individual';
        } elsif ( $class eq 'page' ) {
             $at = 'Page';
             $e = MT::Page->load( $e->id );
        }
        my $tmplate = MT::Template->load ( { blog_id => $e->blog_id } , 
                                           { join => [ 'MT::TemplateMap', 'template_id',
                                           { blog_id => $e->blog_id,
                                             archive_type => $at,
                                             is_preferred => 1 },
                                             { unique => 1 }
                                          ], } );
        $app->trans_error( 'Can\'t load template.' ) unless defined $tmplate;
        my $src = $tmplate->text;
        my $title = encode_html( $e->title );
        my $text  = encode_html( $e->text );
        my $form = "<form method=\"post\" action=\"" . $app->base . $app->uri . "\">";
        $form .= '<input type="hidden" name="id" value="' . $e->id . '" />';
        $form .= '<input type="hidden" name="class" value="' . $class . '" />';
        $form .= '<input type="hidden" name="__mode" value="quicksave" />';
        $form .= '<input type="hidden" name="permalink" value="' . $permalink . '" />';
        my $title_fld = "<input type=\"text\" name=\"title\" value=\"$title\" style=\"width:99%;font-size:110%\" />";
        my $text_fld  = "<textarea name=\"text\" style=\"width:99%;height:200px;font-size:105%\">$text</textarea>";
        $text_fld .= '<p style="text-align:center;margin-top:4px"><input type="submit" value="Save" /></p></form>';
        if ( $class eq 'entry' ) {
            $src =~ s/(<h[1-6].*?>.*?)<\$*MT:*EntryTitle.*?\$*>(.*?<\/h[1-6]>)/$form$1$title_fld$2/is;
            $src =~ s/<\$*MT:*EntryBody.*?\$*>/$text_fld/is;
        } elsif ( $class eq 'page' ) {
            $src =~ s/(<h[1-6].*?>.*?)<\$*MT:*PageTitle.*?\$*>(.*?<\/h[1-6]>)/$form$1$title_fld$2/is;
            $src =~ s/<\$*MT:*PageBody.*?\$*>/$text_fld/is;
        }
        my $html = _build_tmpl( $src, $at, $e, $e->blog, $e->category, $app );
        return $html;
    } else {
        $app->redirect( $app->uri );
    }
}

sub _can_post {
    my ( $user, $blog_id, $entry ) = @_;
    return 0 unless defined $user;
    my $can_post = $user->is_superuser;
    unless ( $can_post ) {
        if ( $entry->class eq 'entry' ) {
            $can_post = $user->permissions( $blog_id )->can_post;
            if ( $can_post ) {
                if ( $user->id != $entry->author_id ) {
                    $can_post = $user->permissions( $blog_id )->can_edit_all_posts;
                }
            }
        } else {
            $can_post = $user->permissions( $blog_id )->can_manage_pages;
        }
    }
    return $can_post;
}

sub _build_tmpl {
    my ( $template, $at, $entry, $blog, $category, $app ) = @_;
    my $ctx = MT::Template::Context->new;
    $ctx->{ __stash }->{ vars }->{ entry_template } = 1 if $at eq 'Individual';
    $ctx->{ __stash }->{ vars }->{ page_template }  = 1 if $at eq 'Page';
    $ctx->stash( 'entry', $entry );
    $ctx->stash( 'page', $entry );
    $ctx->stash( 'blog', $blog );
    $ctx->stash( 'category', $category );
    $ctx->stash( 'blog_id', $blog->id );
    my $build = MT::Builder->new;
    my $tokens = $build->compile( $ctx, $template )
        or return $app->error( $app->translate(
            "Parse error: [_1]", $build->errstr) );
    defined ( my $html = $build->build( $ctx, $tokens ) )
        or return $app->error( $app->translate(
            "Build error: [_1]", $build->errstr) );
    return $html;
}

1;